﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.Text;
using System.IO;
using HeuristicLab.PluginInfrastructure.Manager;
using ICSharpCode.SharpZipLib.Zip;

namespace HeuristicLab.Update.Service {
  public class UpdateLocation : IUpdateLocation {

    #region IUpdateLocation Members

    public IEnumerable<PluginInformation> GetAvailablePluginsByName(string name) {
      using (PluginStoreDataContext ctx = new PluginStoreDataContext()) {
        return from plugin in ctx.Plugins
               where MatchingName(name, plugin.Name)
               select new PluginInformation(plugin.Name, new Version(plugin.Version), plugin.BuildDate);
      }
    }
    public IEnumerable<byte[]> GetPluginFiles(PluginInformation info) {
      using (PluginStoreDataContext ctx = new PluginStoreDataContext()) {
        return from plugin in ctx.Plugins
               from file in ctx.PluginFiles
               where
                 plugin.Name == info.Name &&
                 plugin.BuildDate == info.BuildDate &&
                 plugin.Version == info.Version.ToString() &&
                 file.PluginId == plugin.Id
               select file.Data.ToArray();
      }
    }
    #endregion


    private static bool MatchingName(string name, string pluginName) {
      return name.Equals(pluginName, StringComparison.InvariantCultureIgnoreCase);
    }
  }
}
